#!/usr/bin/env python
# coding: utf-8

import numpy as np                             # numpy v. 1.18.5
from scipy.optimize import curve_fit, fsolve   # scipy v. 1.4.1 
from scipy import sparse                       
from matplotlib import pyplot as plt           # matplotlib v. 3.2.1
import seaborn as sns                          # seaborn v. 0.10.1
import csv                                      


'''Constantes de los elementos Carbono y Níquel y el grafeno'''

Mm_C  = 12.0107             # [Mm_C] = g/mol - Masa molar Carbono

Mm_Ni = 58.6934             # [Mm_Ni] = gr/mol - Masa molar Níquel

n_ϕ1  = 3.82*10**7          # [n_ϕ1] = átomos/μm^2 - Concentración superficial atómica del grafeno


'''Constantes universales'''

NA = 6.022140857*10**23     # [NA] = átomos/mol - Número de Avogadro

R  = 8.314472               # [R] = J/(mol*K) - Constante universal de los gases
   

'''Mallado de la capa de Ni'''

L = 170                     # [L] = nm  -  Espesor del depósito de Ni

Nx = 100                    # Número de intervalos de la capa de Ni

x = np.linspace(0,170,Nx+1) # Vector del mallado de la capa (desde 0 hasta L)

dx = x[1]-x[0]              # [dx] = nm  -  diferencial finito de espacio


'''Parámetros del proceso de recocido y mallado en tiempo'''

T_inicial = 20   # [T_inicial] = °C  -  Temperatura inicial del proceso de recocido. Igual a T_final.

t_subida = 45    # [t_subida] = min  -  tiempo en que el horno se calienta hasta la temperatura máxima

T_max = 900      # [T_max] = °C  -  Temperatura máxima del proceso de recocido

t_T_cte = 20     # [t_T_cte] = min  -  tiempo con el horno a temperatura máxima

t_bajada = 150   # [t_bajada] = min  -  tiempo que tarda el horno en enfriarse hasta temperatura inicial

dt = 0.5         # [dt] = s  -  diferencial finito de tiempo

t_total = (t_subida + t_T_cte + t_bajada)*60 # [t_total] = s  -  Tiempo total de duración del recocido

Nt = int(round(t_total/float(dt)))  # Número de intervalos del mallado
    
t = [i*dt for i in range(Nt+1)]     # Mallado en tiempo de todo el proceso de recocido


'''Se genera el vector de temperaturas del horno en cada
   instante de tiempo del proceso de recocido'''
   
t_subida_s = t_subida*60  # [t_subida_s] = s  -  tiempo en llegar a T_max (en s)

v_T_s = (T_max - T_inicial)/t_subida_s  # [v_T_s] = °C/s  -  taza de incremento de T durante el calentamiento

t_T_cte_s = t_T_cte*60  # [t_T_cte] = s  -  intervalo de tiempo (en s) con temperatura constante del horno

t_bajada_s = t_bajada*60 # [t_bajada_s] = s  -  tiempo en que el horno regresa a la T_inicial del proceso

v_T_b = (T_inicial - T_max)/t_bajada_s  #[v_T_b] = °C/s  -  taza de decrecimiento de T en el enfriamiento


T_max_K = T_max + 273.15    # [T_max_K] = K  -  Temperatura máxima del proceso de recocido (en K)

T_teo = []    # [T_teo] = K  -  temperatura teórica (dada por el horno) en cada intervalo de tiempo

T_teo.append(float(T_inicial) + 273.15)

    
for t_i in t:
        
    if 0 < t_i <= t_subida_s:
            
        T_teo.append(float(T_teo[-1]) + v_T_s*dt)
            
    if t_subida_s < t_i <= t_subida_s + t_T_cte_s:
            
        T_teo.append(T_max + 273.15)
            
    if t_i > t_subida_s + t_T_cte_s:
            
        T_teo.append(float(T_teo[-1]) + v_T_b*dt)


'''Se hace una corrección a la temperatura del horno utilizando la 
   ley de calentamiento/enfriamiento de Newton, con un coeficiente
   α = 0.015 1/s'''
        
T_fit = [20+273.15] # [T_fit] = K  -  temperatura corregida por la Ley de calentamiento/enfriamiento de Newton

T_i = T_fit[0] 

for i in range(1,len(t)):
    
    T_fit.append(T_teo[i] - (T_teo[i]-T_i)*np.exp(-0.015*dt))

    T_i = T_fit[-1]


def ρat_Ni(T):
    
    '''Densidad atómica del Ni (at/μm^3) en función de la temperatura T (K).'''
    
    ρ0_Ni = 9050.137062597534   # [ρ0_Ni] = kg/m^3  -  Densidad volumétrica del Ni a T = 0K.
                                # Obtenida mediante ajuste de los datos de [Abdullaev et al.]
    
    m = -0.47209630857318474    # [m] = kg/m^3/K  -  Taza de cambio de la densidad con la temperatura.
    
    ρ_Ni = (ρ0_Ni + m*T)*1e-15  # [ρ_Ni] = gr/μm^3  -  Densidad volumétrica de Ni en función de T.
    
    ρat = ρ_Ni*NA/Mm_Ni      # [ρat] = at/μm^3  -  Densidad atómica del Ni
    
    return ρat    


def S_CNi(T):                       
    
    '''Solubilidad de C en Ni (at/μm^3) en funcion de la temperatura T (K).''' 
        
    S_wt = np.exp(2.480-4880/T)     # [S_wt] = wt.% - Solubilidad en porcentaje en peso de C 
                                    # repecto a Ni+C [Lander et al.]        
    N_C = S_wt*NA/Mm_C              # Número de átomos de C presentes en S_wt gramos de C
    N_Ni = (100-S_wt)*NA/Mm_Ni      # Número de átomos de Ni en (100 - S_wt) gramos de Ni
    
    V_Ni = N_Ni/ρat_Ni(T)           # [V_Ni] = μm^3  -  Volumen que ocupan los N_Ni átomos de Ni
    
    S = N_C/V_Ni                    # [S] = at/μm^3  -  Solubilidad de C en Ni
    
    return S


def D_CNi_Massaro(T):                                    
    
    '''Coeficiente de difusión de C en Ni (nm^2/s) en función de la temperatura T (K).
       Rango de temperatura: 350 a 700 °C'''
    
    D_CNi = (4.07874*10**-5)*np.exp(-10259.7018/T)*10**14  # [D_CNi] = nm^2/s - Ajuste realizado 
                                                           # a los datos de [Massaro et al.]

    return D_CNi 

    
def D_CNi_Lander(T):                                     
    
    '''Coeficiente de difusión de C en Ni (nm^2/s) en función de la temperatura T (K).
       Intervalo de temperatura: 700 a 1050 °C'''
    
    D_CNi = np.exp(0.909- 20200/T)*10**14       # [D_CNi] = nm^2/s - Tomada de [Lander et al.]

    return D_CNi


T_1=(20200-10259.7018)/(0.909 - np.log(4.07874*10**-5)) - 273.15   # [T_1] = K  -  Temperatura a la cual
                                                                   # el D_CNi de Massaro y Lander coinciden

def D_CNi(T):
    
    '''Coeficiente de difusión de C en Ni (nm^2/s) en función de la temperatura T (K).
       Intervalo de temperatura: 350 a 1050 °C'''
    
    if T <= T_1 + 273.15:
        return D_CNi_Massaro(T)
    
    if T > T_1 + 273.15:
        return D_CNi_Lander(T)
      

def F(T):                 # Parámetro de resolucion, adimensional.
    
    '''
    Parámetro adimensional D_CNi(T)*(dt/dx^2) que se usa en la solución
    numérica del problema de difusion. T se introduce en K.
    '''
    
    return dt*D_CNi(T)/dx**2


def ΔG_Taylor(T):    # [T] = K
    
    '''
    Aproximación de ΔG(T) con serie de Taylor a primer orden utilizando dos valores ajustados
    por Bleu et al. ΔG a 500 y 300 °C. T se introduce en K.
    '''
       
    ΔG_500 = -18800        # [ΔG_500] = J/mol  -  Energía libre de Gibbs de C en Ni para T = 500 °C
    ΔG_300 = -13600        # [ΔG_300] = J/mol  -  Energía libre de Gibbs para T = 300°C
    
    ΔG_T = ΔG_500 + (ΔG_500 - ΔG_300)/(500-300)*(T-500)
    
    return ΔG_T


def θ_eq(T):
    
    '''
    En función de la temperatura T (K) se determina la concentración de equilibrio para
    la capa de carbono segregada. θ_eq = n_ϕ_eq/n_ϕ1
    '''
    
    θ = S_CNi(T)/ρat_Ni(T)*np.exp(-ΔG_Taylor(T)/(R*T))
    
    return θ/(1+θ)


def Continuidad_Dif_Seg(n1, n, T):
    
    '''
    Esta función crea el sistema de ν+2 ecuaciones de continuidad acopladas
    para la parte de difusión y segregación.
    Mediante fsolve serán igualadas a cero para hallar sus raíces (n1).
        
    Parámetros:
    -----------
    n1 : vector de concentraciones al tiempo t+dt (incógnitas)
    n  : vector de concentraciones al tiempo t (conocidas)
    T  : temperatura en K del sistema al tiempo t
    
    Devuelve:
    ---------
    Diccionario con nombre de cada ecuación de continuidad y su respectivo valor.
    '''
    
    # Diccionario de concentraciones conocidas n(t)
    n_d = {}
    for i in range(Nx+1):
        n_d[f'n_{i}'] = n[i]
    
    # Diccionario de concentraciones incógnitas n(t+dt)
    n1_d = {}
    for i in range(Nx+1):
        n1_d[f'n1_{i}'] = n1[i]
        
    
    # Diccionario de ecuaciones de continuidad
    Cont_d = {}
    
    # Condición de frontera en (x=0)
    Cont_d[f'Cont_{0}'] = n1_d[f'n1_{0}'] - S_CNi(T)  
    
    
    # Para no incurrir en valores no permitidos, como ln(n_ϕ=0) = -inf,
    # Se crea el sistema de ecuaciones para distintos casos de los valores críticos de n_ν y n_ϕ
    
    global r   # Bandera para saber cuando se debe activar la condición J_seg.
    
    # Primer caso
    # Si n_ν < 100, no hay forma de que aumente n_ϕ. 
    # Por tanto, se tiene idénticamente n_ϕ = 0.
    if (n_d[f'n_{Nx-1}'] <= 100) and (r == False):
        
        for i in range(1,Nx-1):
            Cont_d[f'Cont_{i}']= (- F(T)*n1_d[f'n1_{i-1}'] 
                                           + 2*(1+F(T))*n1_d[f'n1_{i}']
                                           - F(T)*n1_d[f'n1_{i+1}']
                                           -F(T)*n_d[f'n_{i-1}']
                                           - 2*(1-F(T))*n_d[f'n_{i}']
                                           - F(T)*n_d[f'n_{i+1}'])
        
        Cont_d[f'Cont_{Nx-1}'] = (-F(T)*n1_d[f'n1_{Nx-2}']
                                           + (2+F(T))*n1_d[f'n1_{Nx-1}']
                                           -F(T)*n_d[f'n_{Nx-2}']
                                           -(2-F(T))*n_d[f'n_{Nx-1}'])
        
        Cont_d[f'Cont_{Nx}'] = n1_d[f'n1_{Nx}']
        
        return np.array([Cont_d[f'Cont_{i}'] for i in range(Nx+1)])
    
    
    # Segundo caso
    # Si ya existe la suficiente cantidad de átomos de C en la última celda de Ni, (n_ν > 100):
    # Se toma en cuenta un flujo de segregación pequeño (J_seg ~ 1) que hará n_ϕ > 0.
    # con lo cuál se activará la condición J_seg
    elif (n_d[f'n_{Nx-1}'] > 100) and (r==False):
        
        for i in range(1,Nx-1):
            Cont_d[f'Cont_{i}']= (- F(T)*n1_d[f'n1_{i-1}'] 
                                           + 2*(1+F(T))*n1_d[f'n1_{i}']
                                           - F(T)*n1_d[f'n1_{i+1}']
                                           -F(T)*n_d[f'n_{i-1}']
                                           - 2*(1-F(T))*n_d[f'n_{i}']
                                           - F(T)*n_d[f'n_{i+1}'])
        
        Cont_d[f'Cont_{Nx-1}'] = (-F(T)*n1_d[f'n1_{Nx-2}']
                                           + (2+F(T))*n1_d[f'n1_{Nx-1}']
                                           +2/dx*n1_d[f'n1_{Nx}']
                                           -F(T)*n_d[f'n_{Nx-2}']
                                           -(2-F(T))*n_d[f'n_{Nx-1}']
                                           -2/dx*n_d[f'n_{Nx}'])
        
        Cont_d[f'Cont_{Nx}'] = (n1_d[f'n1_{Nx}'] - dt)
        
        return np.array([Cont_d[f'Cont_{i}'] for i in range(Nx+1)])
                                            
                            
    # Tercer caso
    # Como n_ϕ > 0 ahora r=True y se activa condición de flujo J_seg
    elif (r==True):       
        
        for i in range(1,Nx-1):
            Cont_d[f'Cont_{i}']= (- F(T)*n1_d[f'n1_{i-1}'] 
                                           + 2*(1+F(T))*n1_d[f'n1_{i}']
                                           - F(T)*n1_d[f'n1_{i+1}']
                                           -F(T)*n_d[f'n_{i-1}']
                                           - 2*(1-F(T))*n_d[f'n_{i}']
                                           - F(T)*n_d[f'n_{i+1}'])
        
        Cont_d[f'Cont_{Nx-1}'] = (-F(T)*n1_d[f'n1_{Nx-2}']
                                           + (2+F(T))*n1_d[f'n1_{Nx-1}']
                                           +2/dx*n1_d[f'n1_{Nx}']
                                           -F(T)*n_d[f'n_{Nx-2}']
                                           -(2-F(T))*n_d[f'n_{Nx-1}']
                                           -2/dx*n_d[f'n_{Nx}'])
    
        Cont_d[f'Cont_{Nx}'] = (n1_d[f'n1_{Nx}']
                                           -n_d[f'n_{Nx}']
                                           + dx*F(T)/2*(n1_d[f'n1_{Nx-1}'] + n_d[f'n_{Nx-1}'])*
                                           (ΔG_Taylor(T)/(R*T) 
                                           + np.log((n1_d[f'n1_{Nx}'] + n_d[f'n_{Nx}'])
                                           /(2*n_ϕ1-n1_d[f'n1_{Nx}'] - n_d[f'n_{Nx}'])
                                           *(2*ρat_Ni(T))/(n1_d[f'n1_{Nx-1}'] + n_d[f'n_{Nx-1}']))))
        
        return np.array([Cont_d[f'Cont_{i}'] for i in range(Nx+1)])
        

'''Construcción del método matricial para resolver las ecuaciones de continuidad 
   para la parte de difusión y precipitación.'''

#Matriz tridiagonal A 
A_diagonal = np.zeros(Nx-2)     
A_abajo    = np.zeros(Nx-3)     
A_arriba   = np.zeros(Nx-3)

#Matriz tridiagonal B
B_diagonal = np.zeros(Nx-2)
B_abajo    = np.zeros(Nx-3)
B_arriba   = np.zeros(Nx-3)

#Vectores para C. F. n_0 = n_ν = S_CNi(T)
b   = np.zeros(Nx-2)
b_1 = np.zeros(Nx-2)

def Continuidad_Dif_Precip(n_ini,T):    
    
    #Vectores con la concentracion n en cada intervalo de la capa de Ni 
    n   = n_ini[:-1]*1   # Concentraciones conocidas, n(t). 
                         # El *1 es necesario para que n y n_ini sean variables separadas
    n_1 = np.zeros(Nx)   # Concentraciones incógnitas n(t+dt)
    
    #Condiciones frontera   
    n_1[0]  = S_CNi(T)
    n_1[Nx-1] = S_CNi(T)
    
    #Elementos de b y b_1
    b[0]  = F(T)*n[0]         
    b[-1] = F(T)*n[Nx-1] 
    
    b_1[0]  = -F(T)*n_1[0] 
    b_1[-1] = -F(T)*n_1[Nx-1]  
    
    #Elementos de las tres diagonales de A
    A_diagonal[:] = 2*(1 + F(T))
    A_abajo[:] = -F(T)  
    A_arriba[:] = -F(T)  
    
    #Se genera la matriz A
    A = sparse.diags(diagonals=[A_diagonal, A_abajo, A_arriba],
                    offsets=[0, -1, 1], shape=(Nx-2, Nx-2),format='csr')

    #Elementos de las diagonales de B
    B_diagonal[:] = 2*(1 - F(T))
    B_abajo[:] = F(T)  
    B_arriba[:] = F(T)  

    #Se genera la matriz B
    B = sparse.diags(diagonals=[B_diagonal, B_abajo, B_arriba],
                    offsets=[0, -1, 1], shape=(Nx-2, Nx-2),format='csr')

    #Cambio de variable para que los valores conocidos estén
    #del lado derecho de la Ec. matricial A.n_1 = BB = B.n + b - b_1
    BB = B.dot(n[1:-1]) + b - b_1       

    #Se calcula la solución n_1 
    n_1[1:-1] = sparse.linalg.spsolve(A, BB) 
        
    return n_1


#Matriz que contiene toda la solución n(x,t)
Solucion = np.zeros((Nt+1,Nx+1))

#Condición inicial 
n_ini    = np.zeros(Nx+1)           # n(x>0,0) = 0
n_ini[0] = S_CNi(T_inicial+273.15)  # n(x=0,0) = S_CNi(T)

Solucion[0][:] = n_ini

#Estimación inicial de las concentraciones al tiempo t = dt
n_guess = np.zeros(Nx+1)

i_Tmax = np.argmax(T_fit)


#Se inicializa la bandera r para no activar el flujo de segregación J_seg
r = False   

#Iteración para halalr n(t+dt) en fase de Dif + Seg 
#(hasta terminar la fase de temperatura constante)
for i in range(1,i_Tmax+1):
    
    n_sol = fsolve(Continuidad_Dif_Seg, n_guess, args=(n_ini, T_fit[i]))
    
    if n_sol[-1] > 0:
        r=True
    
    Solucion[i][:] = n_sol
    
    n_ini = n_sol
    n_guess = n_sol

#Iteración para hallar n(t+dt) en fase de Dif + Precip 
#(fase de enfriamiento, hasta terminar proceso de recocido)
for i in range(i_Tmax+1,Nt+1):
          
    n_sol = np.zeros(Nx+1)
    
    n_sol[:-1] = Continuidad_Dif_Precip(n_ini,T_fit[i])
    
    Solucion[i][:-1] = n_sol[:-1]
    
    n_ini = n_sol


# Se guarda la matriz Solucion en un archivo .csv
csv_Sol = open('Solucion'+'.csv','w',newline='')
csv.writer(csv_Sol).writerows(Solucion)
csv_Sol.close()


#Para cargar matriz Solucion perviamente calculada, descomentar la siguiente línea:
#Solucion = np.genfromtxt('Solucion.csv', delimiter=',')


# Se extraen las magnitudes relevantes para la presentación de resultados

#Primera parte: difusión y segregación
t_m     = [t/60 for t in t]            # Vector de tiempo en minutos

T_fit_C = [T - 273.15 for T in T_fit]  # Vector de temperaturas T en °C

i_Tmax = np.argmax(T_fit)

n_ϕ_SLG = Solucion[:i_Tmax+1,-1]/n_ϕ1  # [n_ϕ_SLG] = SLG  -  Evolución temporal de n_ϕ en monocapas de grafeno

n_ϕ_max = np.max(n_ϕ_SLG)              # Número máximo de SLG segregado en la superficie de Ni


#Segunda parte: difusión y precipitación
n_t1 = Solucion[i_Tmax,:-1]            # [n_t1] = μm^-3  -  Perfil de concentración @ t_1 
                                       #                    (terminando fase de T cte)

n_tF = Solucion[-1,:-1]                # [n_tF] = μm^-3  -  Perfil de concentración @ t_F 
                                       #                    (al terminar proceso de recocido)


#Aproximación de la integral para determinar n_p, 
#el número de monocapas precipitadas en una cara del Ni
n_p=0

for i in range(len(x)-1):
    
    n_p += (n_t1[i] - n_tF[i])*dx*1e-3/n_ϕ1/2    #[n_p] = SLG

n_ϕ_p = n_ϕ_max + n_p                  # [n_ϕ_p]   = SLG  -  Monocapas totales (segregación y precipitación) 
                                       #                     de grafeno en superficie del Ni 
n_s_tot = n_ϕ_max + 2*n_p              # [n_s_tot] = SLG  -  Monocapas totales obtenidas en el sistema completo
                                       #                     (interfase Cam-Ni y superficie del Ni)


#Estimación de la equivalencia del C remanente en el interior de Ni
#en monocapas de grafeno (SLG)
n_extra = 0

for i in range(len(x)-1):
    
    n_extra += n_tF[i]*dx*1e-3/n_ϕ1
    

#Resultados:
print('+------------------------------------------------------+')
print('|     Resultados de la solución numérica al modelo     |')
print('+-----------+-------------+---------------+------------+')
print('| Interfase | Segregación | Precipitación | Total, N |')
print(f'|   de Ni   |  n_ϕ [SLG]  |   n_P [SLG]   |   [SLG]    |')
print('+-----------+-------------+---------------+------------+')
print(f'|   Ni-Cam  |      --     |     {n_p:.4f}    |   {n_p:.4f}   |')
print('+-----------+-------------+---------------+------------+')
print(f'|    Ni_ν   |    {n_ϕ_max:.4f}   |     {n_p:.4f}    |   {n_ϕ_p:.4f}   |')
print('+-----------+-------------+---------------+------------+')
print(f'|   Ambas   |    {n_ϕ_max:.4f}   |     {2*n_p:.4f}    |   {n_s_tot:.4f}   |')
print('+-----------+-------------+---------------+------------+')
print('+-----------+-------------+---------------+------------+')
print(f'|  Concentración remanente dentro del Ni: {n_extra:.4f} SLG   |')
print('+------------------------------------------------------+')


#Gráfica de evolución temporal de n_ϕ
fuente = 'Palatino Linotype'
plt.style.use('bmh') 
plt.style.use('seaborn')    #primero ejecutar con 'bmh'

fig1, ax1 = plt.subplots(figsize=(12,8), sharey=True)
ax2 = ax1.twinx()

ax1.plot(t_m[:i_Tmax+1], n_ϕ_SLG, lw=3)
ax2.plot(t_m[:i_Tmax+1], T_fit_C[:i_Tmax+1], color=sns.color_palette()[2], lw=3)

ax1.set_title('''Evolución temporal de $n_ϕ$ y $T$\nhasta terminar fase de difusión y segregación.''', 
              fontname=fuente, fontsize=28)

ax1.set_xlabel("$t$   [min]", fontname=fuente, fontsize=28)
ax1.set_ylabel("$n_ϕ(t)$   [SLG]", fontname=fuente, fontsize=28, color=sns.color_palette()[0])
ax2.set_ylabel("Temperatura, $T$   [°C]", fontname=fuente, fontsize=28, color=sns.color_palette()[2])

ax1.set_xlim(left=None, right=70)
ax1.set_ylim([-0.0005-40*0.1605/930,0.3])
ax2.set_ylim(bottom=-20, top=950)

ax1.set_xticks([0,10,20,30,40,50,60,65])
ax1.set_yticks([0.00,0.05,0.10,0.15,0.20,0.25,0.30])
ax2.set_yticks([20,300,600,900])

ax1.set_xticklabels(['0','10','20','30','40','50','60','65'],fontname=fuente, size=18)
ax1.set_yticklabels(['0.00','0.05','0.10','0.15','0.20','0.25','0.30'],
                    fontname=fuente, size=18, color=sns.color_palette()[0])
ax2.set_yticklabels(['20','300','600','900'],fontname=fuente, size=18, color=sns.color_palette()[2])

plt.savefig('n_phi_T.png')


#Gráficas de perfiles de concentración n(x,t_1) y n(x,t_F)
fig2 = plt.figure(figsize=(12,8))

plt.yscale('log')
plt.plot(x[:-1], n_t1, label='$t_1=65$ min', lw=3)
plt.plot(x[:-1], n_tF, label='$t_F=215$ min', lw=3)

plt.title('Perfiles de concentración de carbono\nen grueso de níquel a los tiempos $t_1$ y $t_F$', 
          fontname=fuente, size=28)
plt.ylabel('Concentración,  $n(x)$   [$\mu$m$^{-3}$]', fontname=fuente, size=28)
plt.xlabel('$x$   [nm]', fontname=fuente, size=28)

plt.xlim([0,168.3])

plt.xticks([0,40,85,130,168.3], ['0','40','85','130','170'], fontname=fuente, size=18)
plt.yticks(fontname=fuente, size=18)
Legend = plt.legend(loc='best', fontsize=24)
plt.setp(Legend.texts, fontname=fuente)

plt.savefig('n_perfiles_t1_tF.png')


#Grafica de evolucion temporal de n 
#en posiciones determinadas de la capa de Ni
n_x1 = Solucion[:,1]           
n_x2 = Solucion[:,50]          
n_x3 = Solucion[:,98]          

D_T=[]
for T in T_fit:
    D_T.append(D_CNi(T))

fig3, (ax1,ax3) = plt.subplots(1,2,figsize=(12,8), sharey=True)
fig3.subplots_adjust(hspace=0.025)
ax2 = ax1.twinx()
ax4 = ax3.twinx()

ax3.set_yscale('log')
ax3.plot(t_m, n_x1, label = f'$n(x_1={x[1]}$ nm)', lw=3, c=sns.color_palette()[0])
ax3.plot(t_m, n_x2, label = f'$n(x_2={x[50]}$ nm)', lw=3, c=sns.color_palette()[1])
ax3.plot(t_m, n_x3, label = f'$n(x_3={x[98]}$ nm)', lw=3, c=sns.color_palette()[3])

ax4.set_yscale('log')
ax4.plot(t_m, D_T, '--', color=sns.color_palette()[2],lw=3)


ax1.set_yscale('log')
ax1.plot(t_m, n_x1, label = f'$n$  a  $x={x[1]}$ nm ', lw=3, c=sns.color_palette()[0])
ax1.plot(t_m, n_x2, label = f'$n$  a  $x={x[50]}$ nm', lw=3, c=sns.color_palette()[1])
ax1.plot(t_m, n_x3, label = f'$n$  a  $x={x[98]}$ nm', lw=3, c=sns.color_palette()[3])

ax2.set_yscale('log')
ax2.plot(t_m, D_T, '--', color=sns.color_palette()[2],lw=3)


ax1.set_xlim([-1,25])
ax3.set_xlim([150,200])

ax1.set_ylim([1,9e9])

fig3.suptitle('''Evolución temporal de $n$ a determinadas\nposiciones $x$, y del coeficiente de difusión $D_{C-Ni}$''',
              fontname=fuente, size=28)
fig3.tight_layout()
fig3.subplots_adjust(top=0.85)
fig3.text(0.5, -0.05, '$t$   [min]', ha='center', fontname=fuente, fontsize=28)

ax1.set_ylabel('Concentración,  $n_x(t)$   [$\mu$m$^{-3}$]', fontname=fuente, size=28)
ax4.set_ylabel('$D_{C-Ni}$   [nm$^2$/s]', fontname=fuente, size=28, color=sns.color_palette()[2])

ax1.set_xticklabels(['','0','5','10','15','20','25'], 
                    fontname=fuente, size=18)
ax1.set_yticklabels(['','10$^0$','10$^2$','10$^4$','10$^6$','10$^8$'],
                    fontname=fuente, size=18)
ax2.set_yticklabels(['','','','','','',''])
ax3.set_xticklabels(['150','160','170','180','190','200'],
                    fontname=fuente, size=18)
ax4.set_yticklabels(['','','10$^{-5}$','10$^{-3}$','10$^{-1}$','10$^1$','10$^3$','10$^5$','10$^7$'],
                   fontname=fuente, size=18, color=sns.color_palette()[2])

Legend = ax1.legend(loc='upper left', fontsize=24)
plt.setp(Legend.texts, fontname=fuente)

plt.savefig('n_xi.png')